----------------------------------------------------------------------------------
--
-- LGHuntingScopeAction.lua
-- Parte del mod Le Gourmet Evolution.
-- Creado por Snake.
--
----------------------------------------------------------------------------------

require "TimedActions/ISBaseTimedAction"

LGHuntingScopeAction = ISBaseTimedAction:derive("LGHuntingScopeAction");

LGHuntingScopeAction.swingTimer = 0;
LGHuntingScopeAction.shooted = 0;

-- Fenris: new function checks if the item is a ORGM rifle, returns bool.
LGHuntingScopeAction.isORGMRifle = function(item)
    if item:getModule() == 'ORGM' then
        local gunData = ORGM.Firearm.getData(item)
        if gunData.category == ORGM.RIFLE then return true end
    end
    return false
end

-- Fenris: new table holding totalpercent bonuses for LGHuntingScopeAction:start()
-- much cleaner then having them all hardcoded into the function.
-- Since actual scopes in ORGM are rarely used and differ in power (x8 Max vs x40)
-- the bonus given to them are slightly different (for balancing)
LGHuntingScopeAction.ScopeTable ={
    ["x4Scope"] = 5,
    ["x8Scope"] = 10,
    ["x20Scope"] = 12,
    ["x40Scope"] = 14,
    ["StealthTacticalScope"] = 16,
    ["StealthTacticalScope2"] = 20,
    -- orgm
    ["2xScope"] = 4,
    ["4xScope"] = 8,
    ["8xScope"] = 16,
}


function LGHuntingScopeAction:isValid()
	return true;
end


function LGHuntingScopeAction:update()
	LGHuntingScopeAction.swingTimer = LGHuntingScopeAction.swingTimer - 1;
    if LGHuntingScopeAction.swingTimer == 0 then
        self.character:PlayAnim("Idle");
--		self.character:playSound(reloadable.clickSound, true);
--		self.character:getEmitter():playSound("brokeCR", false);
--		addSound(self.object, self.object:getX(), self.object:getY(), self.object:getZ(), 20, 10);
    end
end


function LGHuntingScopeAction:start()
	local Primary = self.character:getPrimaryHandItem();
    self.character:faceLocation(self.htree2:getX(), self.htree2:getY())
    self.weapon = self.character:getPrimaryHandItem()
    self.weapon:setJobType(getText("ContextMenu_BreakOpen"));
    self.weapon:setJobDelta(0.0);
    self.character:PlayAnimUnlooped("Attack_" .. self.weapon:getSwingAnim())
	local _hand = self.character:getPrimaryHandItem();
	local data = _hand:getModData();
	local reloadable = ReloadUtil:getReloadableWeapon(_hand, self.character)
	if data.roundChambered ~= nil and data.roundChambered > 0 and data.roundChambered ~= "x" then
		reloadable:fireShot(_hand, ISReloadManager.getDifficulty())

		if (instanceof(Primary, "HandWeapon") and Primary:getCondition() > 0) then
            -- Fenris: added call to LGHuntingScopeAction.isORGMRifle(Primary)
			if Primary:getType() == "VarmintRifle" or Primary:getType() == "HuntingRifle" or LGHuntingScopeAction.isORGMRifle(Primary) then
				local sound = nil;
				local SoundMod = nil;
				local RadiusMod = nil;
				if Primary:getType() == "VarmintRifle" then
					sound = "FirearmVarmintShot";
					SoundMod = 200 * 0.05;
					RadiusMod = 150 * 0.1;
				else
					sound = "FirearmHunterShot";
					SoundMod = 20 * 0.1;
					RadiusMod = 150 * 0.1;
				end
				if Primary:getCanon() ~= nil then
					local Canon = Primary:getCanon();
					if Canon:getType() == "Silencer3" or Canon:getType() == "Silencer6" then
						self.character:playSound('WPARSilencedShot', true);
						addSound(self.character, self.character:getX(), self.character:getY(), self.character:getZ(), RadiusMod, SoundMod);
					else
                        -- Fenris: Changed Here to play _hand:getSwingSound().. so we use the weapons current sound effect.
                        -- This work fine with ORGM Silencers mod too.
						self.character:playSound(_hand:getSwingSound(), true);
						addSound(self.character, self.character:getX(), self.character:getY(), self.character:getZ(), _hand:getSoundRadius(), _hand:getSoundVolume());
					end
				else
					self.character:playSound(_hand:getSwingSound(), true);
					addSound(self.character, self.character:getX(), self.character:getY(), self.character:getZ(), _hand:getSoundRadius(), _hand:getSoundVolume());
				end
			end
		end
		--addSound(self.character, self.character:getX(), self.character:getY(), self.character:getZ(), 20, 10);


		LGHuntingScopeAction.shooted = 1;
	else
		self.character:playSound(reloadable.clickSound, true);
		LGHuntingScopeAction.shooted = 0;
	end
    self.character:SetAnimFrame(0, false);
    self.character:getSpriteDef():setFrameSpeedPerFrame(0.1);
	LGHuntingScopeAction.swingTimer = 15;
	if ZombRand(Primary:getConditionLowerChance()) == 0 then
		Primary:setCondition(Primary:getCondition() - 1)
		ISWorldObjectContextMenu.checkWeapon(self.character);
	end
	if LGHuntingScopeAction.shooted == 1 then
		if self.capture2 ~= "nothing" then
			local totalpercent = 10;
			local variable = 0;
			local perklvl = self.character:getPerkLevel(Perks.Aiming);
			if perklvl > 0 then
				variable = perklvl * 4;
				totalpercent = totalpercent + variable;
			end
			if self.character:HasTrait("ShortSighted") then
				totalpercent = totalpercent - 10;
			elseif self.character:HasTrait("EagleEyed") then
				totalpercent = totalpercent + 10;
			end
			if (instanceof(Primary, "HandWeapon") and Primary:getCondition() > 0) then
                -- Fenris: added call to LGHuntingScopeAction.isORGMRifle(Primary)
				if Primary:getType() == "VarmintRifle" or Primary:getType() == "HuntingRifle" or LGHuntingScopeAction.isORGMRifle(Primary) then
                    if Primary:getScope() ~= nil then
                        -- Fenris: check the LGHuntingScopeAction.ScopeTable to get our bonus to totalpercent
                        local scopeType = Primary:getScope():getType()
                        totalpercent = totalpercent + (LGHuntingScopeAction.ScopeTable[scopeType] or 0)
        			end
				end
			end

			if not self.character:HasTrait("Desensitized") then
				local paniclvl = self.character:getStats():getPanic();
				if paniclvl > 0 then
					local panicvariable = 0;
					if paniclvl > 0 and paniclvl < 25 then
						panicvariable = 5;
					elseif paniclvl > 25 and paniclvl < 50 then
						panicvariable = 10;
					elseif paniclvl > 50 and paniclvl < 75 then
						panicvariable = 15;
					elseif paniclvl > 75 and paniclvl < 100 then
						panicvariable = 20;
					end
					totalpercent = totalpercent - panicvariable;
				end
			end
			if totalpercent <= 0 then
				totalpercent = 1;
			end
			if totalpercent >= 100 then
				totalpercent = 100;
			end
			if ZombRand(1, 101) <= totalpercent then
				local AnimalSVolume = 0;
				local AnimalSRadius = 0;
				local AnimalSize = 50;
				local AnimalWeigthChange = 1;
				getSoundManager():PlayWorldSound("HeadSmash", self.htree2:getSquare(), 0, 5, 1, false);
				if self.capture2 == "Base.DeadDeer" then
					getSoundManager():PlayWorldSound("DeerDead", self.htree2:getSquare(), 0, 100, 20, false);
					AnimalSVolume = 20;
					AnimalSRadius = 50;
					self.character:playSound("DeerDead", false);
					AnimalSize = ZombRand(45,81);
					AnimalWeigthChange = 1.1;
				elseif self.capture2 == "Base.DeadBear" then
					getSoundManager():PlayWorldSound("BearDead", self.htree2:getSquare(), 0, 150, 100, true);
					AnimalSVolume = 50;
					AnimalSRadius = 100;
					AnimalSize = ZombRand(40,66);
					AnimalWeigthChange = 2.2;
					self.character:playSound("BearDead", false);
				elseif self.capture2 == "Base.DeadPork" then
					getSoundManager():PlayWorldSound("BoarDead", self.htree2:getSquare(), 0, 150, 100, true);
					AnimalSVolume = 50;
					AnimalSRadius = 100;
					AnimalSize = ZombRand(50,96);
					AnimalWeigthChange = 1.1;
					self.character:playSound("BoarDead", false);
				elseif self.capture2 == "Base.DeadCow" then
					getSoundManager():PlayWorldSound("CowDead", self.htree2:getSquare(), 0, 150, 100, true);
					AnimalSVolume = 50;
					AnimalSRadius = 100;
					AnimalSize = ZombRand(65,126);
					AnimalWeigthChange = 1.8;
					self.character:playSound("CowDead", false);
				elseif self.capture2 == "Base.DeadBison" then
					getSoundManager():PlayWorldSound("BisonDead", self.htree2:getSquare(), 0, 150, 100, true);
					AnimalSVolume = 50;
					AnimalSRadius = 100;
					AnimalSize = ZombRand(75,135);
					AnimalWeigthChange = 1.8;
					self.character:playSound("BisonDead", false);
				elseif self.capture2 == "Base.DeadMoose" then
					getSoundManager():PlayWorldSound("MooseDead", self.htree2:getSquare(), 0, 150, 100, true);
					AnimalSVolume = 50;
					AnimalSRadius = 100;
					AnimalSize = ZombRand(40,70);
					AnimalWeigthChange = 2.2;
					self.character:playSound("MooseDead", false);
				end

				local zone = self:getScavengeZone();
				local scavengeZoneNumber = ZombRand(5,15) + self.scavengeZoneIncrease;
				local actualzone = ISScavengeAction.getScavengingZone(self.htree2:getX(), self.htree2:getY());

				local Animalgived = InventoryItemFactory.CreateItem(self.capture2);
				Animalgived:setBaseHunger(- AnimalSize / 100);
				Animalgived:setHungChange(- AnimalSize / 100);
				Animalgived:setActualWeight(AnimalSize * AnimalWeigthChange);
				Animalgived:setCustomWeight(true);
				addSound(self.htree2, self.htree2:getX(), self.htree2:getY(), self.htree2:getZ(), AnimalSRadius, AnimalSVolume);
				self.htree2:getSquare():AddWorldInventoryItem(Animalgived, self.htree2:getX() - math.floor(self.htree2:getX()), self.htree2:getY() - math.floor(self.htree2:getY()), self.htree2:getZ() - math.floor(self.htree2:getZ()));
				local oldXp = self.character:getXp():getXP(Perks.Aiming);
				self.character:getXp():AddXP(Perks.Aiming, (perklvl +1) * ZombRand(5,11));
				local xp = self.character:getXp():getXP(Perks.Aiming) - oldXp;
				self.character:setHaloNote(getText("IGUI_perks_Aiming").."+"..tonumber(string.format("%." .. (2 or 0) .. "f", xp)), 0,255,0,300);

				if not zone then -- register the new zone
					zone = getWorld():registerZone(scavengeZoneNumber .. "", "PlantScavenge",self.htree2:getX() - 20, self.htree2:getY() - 20, 0, 40, 40);
					zone:setLastActionTimestamp(getGametimeTimestamp());
					local plantLeft = tonumber(zone:getName());
					local totalnumber = plantLeft - 3;
					if totalnumber <= 0 then
						zone:setName(0 .. "");
					else
						zone:setName(totalnumber .. "");
					end
				else
					local plantLeft = tonumber(zone:getName());
					zone:setName(plantLeft - 5 .. "");
					zone:setLastActionTimestamp(getGametimeTimestamp());
					if actualzone:getName() <= "0" then
						zone:setName(0 .. "");
					end
				end
				if isClient() then zone:sendToServer() end
			else
				local saying = ZombRand(3);
				if saying == 0 then
					self.character:Say(getText("Tooltip_AnimalGone"));
				elseif saying == 1 then
					self.character:Say(getText("Tooltip_AnimalGone2"));
				else
					self.character:Say(getText("Tooltip_AnimalGone3"));
				end
				local oldXp = self.character:getXp():getXP(Perks.Aiming);
				self.character:getXp():AddXP(Perks.Aiming, 1);
				local xp = self.character:getXp():getXP(Perks.Aiming) - oldXp;
				self.character:setHaloNote(getText("IGUI_perks_Aiming").."+"..tonumber(string.format("%." .. (2 or 0) .. "f", xp)), 0,255,0,300);
			end
		else
		end
	else
	end
end


function LGHuntingScopeAction:stop()
    ISBaseTimedAction.stop(self);
end


function LGHuntingScopeAction:perform()
	self.character:PlayAnim("Idle");
	ISBaseTimedAction.perform(self);
end

function LGHuntingScopeAction:getScavengeZone()
    local zones = getWorld():getMetaGrid():getZonesAt(self.htree2:getX(), self.htree2:getY(), 0);
    if zones then
        for i=0,zones:size()-1 do
            if zones:get(i):getType() == "PlantScavenge" then
                return zones:get(i);
            end
        end
    end
end


function LGHuntingScopeAction:new(player, htree2, time, capture2)
	local o = {}
	setmetatable(o, self)
	self.__index = self
	o.character = player;
	o.htree2 = htree2;
	o.capture2 = capture2;
	o.stopOnWalk = true;
	o.stopOnRun = true;
	o.maxTime = time;

    -- sandbox settings
    o.scavengeZoneIncrease = 0;
    if SandboxVars.NatureAbundance == 1 then -- very poor
        o.scavengeZoneIncrease = -5;
    elseif SandboxVars.NatureAbundance == 2 then -- poor
        o.scavengeZoneIncrease = -2;
    elseif SandboxVars.NatureAbundance == 4 then -- abundant
        o.scavengeZoneIncrease = 2;
    elseif SandboxVars.NatureAbundance == 5 then -- very abundant
        o.scavengeZoneIncrease = 5;
    end
	return o
end
