--***********************************************************
--**                    THE INDIE STONE                    **
--***********************************************************
require "TimedActions/ISScavengeAction"
require "TimedActions/ISBaseTimedAction"

-- random a plant/insect, more chance in deep forest
function ISScavengeAction:scavenge()
    self.character:getStats():setEndurance(self.character:getStats():getEndurance() - 0.004)
    local zone = self:getScavengeZone();
    local scavengeZoneNumber = ZombRand(5,15) + self.scavengeZoneIncrease;
    if scavengeZoneNumber <= 0 then
        scavengeZoneNumber = 1;
    end
    if zone then
        local plantLeft = tonumber(zone:getName());
        if getGametimeTimestamp() - zone:getLastActionTimestamp() > 50000 then
            --            print("refill zone");
            zone:setName(scavengeZoneNumber .. "");
            zone:setOriginalName(scavengeZoneNumber .. "");
        elseif plantLeft == 0 then return; end
    end

    local item = self:getPlant();
    if item ~= nil then
        self:addOrDropItems(item.type, item.count);
        self.character:getXp():AddXP(Perks.PlantScavenging, 7);
    elseif ZombRand(9) == 0 then -- give some xp even for a fail
        self.character:getXp():AddXP(Perks.PlantScavenging, 1);
    end

    local goods = self:getForestGoods();
    if goods ~= nil then
        self:addOrDropItems(goods.type, goods.count);
        self.character:getXp():AddXP(Perks.PlantScavenging, 3);
    elseif ZombRand(9) == 0 then -- give some xp even for a fail
        self.character:getXp():AddXP(Perks.PlantScavenging, 1);
    end
	
	local metals = nil;
	local primdetector = self.character:getPrimaryHandItem();
	if primdetector ~= nil and primdetector:getType() == "MetalDetector" and primdetector:isActivated() and primdetector:getUsedDelta() > 0 then
		metals = self:getMetals();
		if metals ~= nil then
			self:addOrDropItems(metals.type, metals.count);
			self.character:getXp():AddXP(Perks.PlantScavenging, 3);
		elseif ZombRand(9) == 0 then -- give some xp even for a fail
			self.character:getXp():AddXP(Perks.PlantScavenging, 1);
		end
	end
	
	local medicinal = nil;
    if self.character:getKnownRecipes():contains("Herbalist") then
        medicinal = self:getMedicinalHerbs();
        if medicinal ~= nil then
            self:addOrDropItems(medicinal.type, medicinal.count);
            self.character:getXp():AddXP(Perks.PlantScavenging, 5);
        elseif ZombRand(9) == 0 then -- give some xp even for a fail
            self.character:getXp():AddXP(Perks.PlantScavenging, 1);
        end
    end

    local seedcat = nil;
    if self.character:getKnownRecipes():contains("Herbalist") then
        seedcat = self:getSeeds();
        if seedcat ~= nil then
            self:addOrDropItems(seedcat.type, seedcat.count);
            self.character:getXp():AddXP(Perks.PlantScavenging, 5);
        elseif ZombRand(9) == 0 then -- give some xp even for a fail
            self.character:getXp():AddXP(Perks.PlantScavenging, 1);
        end
    end

    if item or goods or metals or medicinal or seedcat then -- got something
        if not zone then -- register the new zone
            --            print("register new zone");
            zone = getWorld():registerZone(scavengeZoneNumber .. "", "PlantScavenge",self.x - 20, self.y - 20, 0, 40, 40);
            zone:setLastActionTimestamp(getGametimeTimestamp());
        else -- update the plant available
            local plantLeft = tonumber(zone:getName());
            zone:setName(plantLeft - 1 .. "");
--            print("got somethiong",plantLeft)
            --            print("plant left : " .. plantLeft - 1);
            zone:setLastActionTimestamp(getGametimeTimestamp());
        end
        if isClient() then zone:sendToServer() end
        return true;
    end
    return false;
end


function ISScavengeAction:getMetals()
    if not self.options["Metals"] then
        return nil;
    end
    local level = self.character:getPerkLevel(Perks.PlantScavenging);
    local baseChance = 8;
    if self.zone:getType() == "DeepForest" then
        baseChance = 15;
    end
    baseChance = baseChance + (4 * level);
    baseChance = baseChance + self.bonusFindingChance;
    if baseChance > (30 + self.bonusFindingChance) then
        baseChance = 30 + self.bonusFindingChance;
    end

    local possibilities = {};
    -- check for all the scavenging possibilities
    for i,v in ipairs(scavenges.Metals) do
        if v.skill <= level then
            table.insert(possibilities, v);
        end
    end

    --    baseChance = 100;

    -- you now get more than 1 plants
    if #possibilities > 0 and ZombRand(100) < baseChance then
        local result = {};
        local item = possibilities[ZombRand(#possibilities) + 1];
        local count = ZombRand(item.minCount, item.maxCount) + (level);
        if count > item.maxCount then
            count = item.maxCount;
        end
        result.type = item.type;
        result.count = count;
        return result;
    end

    return nil;
end

function ISScavengeAction:getSeeds()
    if not self.options["Seeds"] then
        return nil;
    end
    -- no foraging from november to february
    if (getGameTime():getMonth() + 1) >= 11 or (getGameTime():getMonth() + 1) <= 2 then
        return nil;
    end

    local level = self.character:getPerkLevel(Perks.PlantScavenging);
    local baseChance = 3;
    if self.zone:getType() == "DeepForest" then
        baseChance = 7;
    end
    baseChance = baseChance + (2 * level);
    baseChance = baseChance + self.bonusFindingChance;

    if baseChance > (23 + self.bonusFindingChance) then
        baseChance = 23 + self.bonusFindingChance;
    end

    local possibilities = {};
    -- check for all the scavenging possibilities
    for i,v in ipairs(scavenges.Seeds) do
        if v.skill <= level then
            table.insert(possibilities, v);
        end
    end

    --    baseChance = 100;

    -- you now get more than 1 plants
    if #possibilities > 0 and ZombRand(100) < baseChance then
        local result = {};
        local item = possibilities[ZombRand(#possibilities) + 1];
        local count = ZombRand(item.minCount, item.maxCount) + (level / 2);
        if count > item.maxCount then
            count = item.maxCount;
        end
        result.type = item.type;
        result.count = count;
        return result;
    end

    return nil;
end